@@ -18,10 +18,9 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-/* $Id: oasys.c,v 1.29 1991/08/19 23:03:07 pesch Exp $ */
+/* $Id: oasys.c,v 1.34 1991/09/17 05:09:21 grossman Exp $ */
 
 #define UNDERSCORE_HACK 1
-#define offsetof(type, identifier) (size_t) &(((type *) 0)->identifier) 
 #include <ansidecl.h>
 #include <sysdep.h>
 
@@ -30,6 +29,13 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "oasys.h"
 #include "liboasys.h"
 
+/* XXX - FIXME.  offsetof belongs in the system-specific files in
+   ../include/sys. */
+/* Define offsetof for those systems which lack it */
+
+#ifndef offsetof
+#define offsetof(type, identifier) (size_t) &(((type *) 0)->identifier) 
+#endif
 
 /* Read in all the section data and relocation stuff too */
 PROTO(static boolean,oasys_slurp_section_data,(bfd *CONST abfd));
@@ -156,6 +162,10 @@ DEFUN(oasys_slurp_symbol_table,(abfd),
 	    dest->section = (asection *)NULL;
 	    dest->flags = BSF_FORT_COMM;
 	    break;
+	  default:
+	    dest = dest_defined--;
+	    BFD_ASSERT(0);
+	    break;
 	  }
 	  dest->name = string_ptr;
 	  dest->the_bfd = abfd;
@@ -180,7 +190,6 @@ DEFUN(oasys_slurp_symbol_table,(abfd),
     }
   }
   return true;
-
 }
 
 static unsigned int
